'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var React = require('react');
var prefixClsFn = require('./utils').prefixClsFn;

var Nav = React.createClass({
  displayName: 'Nav',

  mixins: [require('./InkBarMixin')],

  getInitialState: function getInitialState() {
    return {
      next: false,
      offset: 0,
      prev: false
    };
  },

  _getTabs: function _getTabs() {
    var _this = this;

    var props = this.props;
    var children = props.panels;
    var activeKey = props.activeKey;
    var rst = [];
    var prefixCls = props.prefixCls;

    React.Children.forEach(children, function (child) {
      var key = child.key;
      var cls = activeKey === key ? prefixClsFn(prefixCls, 'tab-active') : '';
      cls += ' ' + prefixClsFn(prefixCls, 'tab');
      var events = {};
      if (child.props.disabled) {
        cls += ' ' + prefixClsFn(prefixCls, 'tab-disabled');
      } else {
        events = {
          onClick: _this.handleTabClick.bind(_this, key)
        };
      }
      rst.push(React.createElement(
        'div',
        _extends({}, events, { className: cls, key: key,
          ref: 'tab' + key, 'data-active': activeKey === key }),
        React.createElement(
          'a',
          null,
          child.props.tab
        )
      ));
    });

    return rst;
  },

  handleTabClick: function handleTabClick(key) {
    this.props.handleTabClick(key);
  },

  componentDidMount: function componentDidMount() {
    this.componentDidUpdate();
  },

  componentDidUpdate: function componentDidUpdate() {
    var navNode = React.findDOMNode(this.refs.nav);
    var navNodeWidth = navNode.offsetWidth;
    var navWrapNode = React.findDOMNode(this.refs.navWrap);
    var navWrapNodeWidth = navWrapNode.offsetWidth;
    var state = this.state;
    var offset = state.offset;
    if (navWrapNodeWidth - offset < navNodeWidth) {
      if (!state.next) {
        this.setState({
          next: true
        });
      }
    } else {
      var minOffset = navWrapNodeWidth - navNodeWidth;
      if (minOffset < 0 && minOffset > offset) {
        if (state.next) {
          this.setState({
            next: false
          });
        }
        this.setOffset(minOffset);
        offset = minOffset;
      }
    }
    if (offset < 0) {
      if (!state.prev) {
        this.setState({
          prev: true
        });
      }
    } else if (state.prev) {
      this.setState({
        prev: false
      });
    }
  },

  setOffset: function setOffset(offset) {
    offset = Math.min(0, offset);
    this.setState({
      offset: offset
    });
  },

  prev: function prev() {
    var navWrapNode = React.findDOMNode(this.refs.navWrap);
    var navWrapNodeWidth = navWrapNode.offsetWidth;
    var state = this.state;
    var offset = state.offset;
    this.setOffset(offset + navWrapNodeWidth);
  },

  next: function next() {
    var navWrapNode = React.findDOMNode(this.refs.navWrap);
    var navWrapNodeWidth = navWrapNode.offsetWidth;
    var state = this.state;
    var offset = state.offset;
    this.setOffset(offset - navWrapNodeWidth);
  },

  render: function render() {
    var props = this.props;
    var state = this.state;
    var prefixCls = props.prefixCls;
    var tabs = this._getTabs();
    var tabMovingDirection = props.tabMovingDirection;
    var inkBarClass = prefixClsFn(prefixCls, 'ink-bar');
    if (tabMovingDirection) {
      inkBarClass += ' ' + prefixClsFn(prefixCls, 'ink-bar-transition-' + tabMovingDirection);
    }
    var nextButton, prevButton;

    if (state.prev) {
      prevButton = React.createElement(
        'span',
        {
          onClick: this.prev,
          unselectable: 'unselectable',
          className: prefixClsFn(prefixCls, 'tab-prev') },
        React.createElement('span', { className: prefixClsFn(prefixCls, 'tab-prev-icon') })
      );
    }

    if (state.next) {
      nextButton = React.createElement(
        'span',
        {
          onClick: this.next,
          unselectable: 'unselectable',
          className: prefixClsFn(prefixCls, 'tab-next') },
        React.createElement('span', { className: prefixClsFn(prefixCls, 'tab-next-icon') })
      );
    }

    return React.createElement(
      'div',
      { className: prefixClsFn(prefixCls, 'nav-container'), ref: 'container' },
      prevButton,
      nextButton,
      React.createElement(
        'div',
        { className: prefixClsFn(prefixCls, 'nav-wrap'), ref: 'navWrap' },
        React.createElement(
          'div',
          { className: prefixClsFn(prefixCls, 'nav-scroll') },
          React.createElement(
            'div',
            { className: prefixClsFn(prefixCls, 'nav'), ref: 'nav', style: { left: state.offset } },
            React.createElement('div', { className: inkBarClass, ref: 'inkBar' }),
            tabs
          )
        )
      )
    );
  }
});

module.exports = Nav;