'use strict';

var React = require('react');
var KeyCode = require('./KeyCode');
var TabPane = require('./TabPane');
var Nav = require('./Nav');
var CSSTransitionGroup = require('rc-css-transition-group');
function noop() {}
var utils = require('./utils');
var prefixClsFn = utils.prefixClsFn;

var Tabs = React.createClass({
  displayName: 'Tabs',

  getInitialState: function getInitialState() {
    var props = this.props;
    var activeKey;
    if ('activeKey' in props) {
      activeKey = props.activeKey;
    } else if ('defaultActiveKey' in props) {
      activeKey = props.defaultActiveKey;
    } else {
      React.Children.forEach(props.children, function (child) {
        if (!activeKey) {
          activeKey = child.key;
        }
      });
    }
    //this.handleKeyDown = this.handleKeyDown.bind(this);
    //this.handleTabDestroy = this.handleTabDestroy.bind(this);
    // cache panels
    this.renderPanels = {};
    return { activeKey: activeKey };
  },

  setActiveKey: function setActiveKey(activeKey) {
    var currentActiveKey = this.state.activeKey;
    if (!currentActiveKey) {
      this.setState({
        activeKey: activeKey
      });
    } else {
      var left;
      React.Children.forEach(this.props.children, function (c) {
        if (left !== undefined) {
          return;
        }
        var key = c.key;
        if (currentActiveKey === key) {
          left = false;
        } else if (activeKey === key) {
          left = true;
        }
      });
      var tabMovingDirection = left === true ? 'left' : left === false ? 'right' : '';
      this.setState({
        activeKey: activeKey,
        tabMovingDirection: tabMovingDirection
      });
    }
  },

  componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
    if ('activeKey' in nextProps) {
      this.setActiveKey(nextProps.activeKey);
    }
  },

  handleTabDestroy: function handleTabDestroy(key) {
    delete this.renderPanels[key];
  },

  _getNextActiveKey: function _getNextActiveKey() {
    var activeKey = this.state.activeKey;
    var children = [];
    React.Children.forEach(this.props.children, function (c) {
      if (!c.props.disabled) {
        children.push(c);
      }
    });
    var length = children.length;
    var ret = length && children[0].key;
    children.forEach(function (child, i) {
      if (child.key === activeKey) {
        if (i === length - 1) {
          ret = children[0].key;
        } else {
          ret = children[i + 1].key;
        }
      }
    });
    return ret;
  },

  _getPreviousActiveKey: function _getPreviousActiveKey() {
    var activeKey = this.state.activeKey;
    var children = [];
    React.Children.forEach(this.props.children, function (c) {
      if (!c.props.disabled) {
        children.unshift(c);
      }
    });
    var length = children.length;
    var ret = length && children[length - 1].key;
    children.forEach(function (child, i) {
      if (child.key === activeKey) {
        if (i === length - 1) {
          ret = children[0].key;
        } else {
          ret = children[i + 1].key;
        }
      }
    });
    return ret;
  },

  _getTabPanes: function _getTabPanes() {
    var _this = this;

    var activeKey = this.state.activeKey;
    var children = this.props.children;
    var newChildren = [];
    var renderPanels = this.renderPanels;

    React.Children.forEach(children, function (child) {
      var key = child.key;
      var active = activeKey === key;
      if (active || renderPanels[key]) {
        child = active ? child : renderPanels[key];
        renderPanels[key] = React.cloneElement(child, {
          active: active,
          onDestroy: _this.handleTabDestroy.bind(_this, key),
          rootPrefixCls: _this.props.prefixCls
        });
        newChildren.push(renderPanels[key]);
      } else {
        // lazy load
        newChildren.push(React.createElement(TabPane, { active: false,
          key: key,
          rootPrefixCls: _this.props.prefixCls }));
      }
    });

    return newChildren;
  },

  handleTabClick: function handleTabClick(key) {
    this.props.onTabClick(key);
    if (this.state.activeKey !== key) {
      this.setActiveKey(key);
      this.props.onChange(key);
    }
  },

  handleKeyDown: function handleKeyDown(e) {
    if (e.target !== React.findDOMNode(this)) {
      return;
    }
    var eventKeyCode = e.keyCode;
    switch (eventKeyCode) {
      case KeyCode.RIGHT:
      case KeyCode.DOWN:
        e.preventDefault();
        var nextKey = this._getNextActiveKey();
        this.handleTabClick(nextKey);
        break;
      case KeyCode.LEFT:
      case KeyCode.UP:
        e.preventDefault();
        var previousKey = this._getPreviousActiveKey();
        this.handleTabClick(previousKey);
        break;
      default:
    }
  },

  render: function render() {
    var props = this.props;
    var effect = this.props.effect;
    var prefixCls = props.prefixCls;
    var cls = prefixCls;
    var tabMovingDirection = this.state.tabMovingDirection;
    if (props.className) {
      cls += ' ' + props.className;
    }
    var tabPanes = this._getTabPanes();
    if (effect) {
      tabPanes = React.createElement(
        CSSTransitionGroup,
        { showProp: 'active',
          exclusive: true,
          transitionName: prefixClsFn(prefixCls, 'effect-' + (tabMovingDirection || 'left')) },
        tabPanes
      );
    }
    return React.createElement(
      'div',
      { className: cls, tabIndex: '0', onKeyDown: this.handleKeyDown },
      React.createElement(Nav, { prefixCls: prefixCls,
        handleTabClick: this.handleTabClick,
        tabMovingDirection: tabMovingDirection,
        panels: this.props.children,
        activeKey: this.state.activeKey }),
      React.createElement(
        'div',
        { className: prefixClsFn(prefixCls, 'content') },
        tabPanes
      )
    );
  }
});

Tabs.defaultProps = {
  prefixCls: 'rc-tabs',
  onChange: noop,
  onTabClick: noop
};

Tabs.TabPane = TabPane;

module.exports = Tabs;